function init()
    RegisterTool("onepunchman", "One Punch Man", "MOD/vox/Onepunchman.vox")
	SetBool("game.tool.onepunchman.enabled", true)
    
    punchSound = LoadSound("MOD/snd/punch.ogg")

	swingTimer = 0
    stepsize = 0.1
    punchedShape = nil
    punchVelocity = 100
    voxelLimit = 1000000
end

function tick(dt)
	if GetString("game.player.tool") == "onepunchman" and GetBool("game.player.canusetool") then
        if punchedShape then
            local body = GetShapeBody(punchedShape)
            local shapes = GetBodyShapes(body)
            local voxelCount = 0
            for i=1, #shapes do
                voxelCount = voxelCount + GetShapeVoxelCount(shapes[i])
            end

            if voxelCount < voxelLimit then
                SetBodyVelocity(body, VecScale(TransformToParentVec(GetCameraTransform(), Vec(0,0,-1)), punchVelocity))
                PlaySound(punchSound, GetCameraTransform().pos, 1)
            end

            punchedShape = nil
        end

        if InputPressed("lmb") and swingTimer <= 0 then
            local ct = GetCameraTransform()
            local fwd = TransformToParentVec(ct, Vec(0,0,-1))
            local hit, dist, normal, shape = QueryRaycast(ct.pos, fwd, 10)
            
            if hit then
                local body = GetShapeBody(shape)
                if IsBodyDynamic(body) then
                    punched = true
                    punchedShape = shape
                    swingTimer = 0.8
                else
                    local min, max = nil, nil

                    local shapes = GetBodyShapes(body)
                    local voxelCount = 0
                    for i=1, #shapes do
                        voxelCount = voxelCount + GetShapeVoxelCount(shapes[i])
                    end

                    if voxelCount < voxelLimit then
                        min, max = GetBodyBounds(body)
                    elseif GetShapeVoxelCount(shape) < voxelLimit then
                        min, max = GetShapeBounds(shape)
                    end

                    if min and max then
                        local center = VecLerp(min, max, 0.5)
                        local length1 = max[1] - min[1]
                        local length2 = max[2] - min[2]
                        local length3 = max[3] - min[3]
                        local count1 = length1 / stepsize
                        local count2 = length2 / stepsize
                        local count3 = length3 / stepsize
                        local faces = {
                            {p = function(i, j) hole(VecAdd(min, Vec(length1 * (i/count1), length2 * (j/count2), 0)), center) end, c1 = count1, c2 = count2},
                            {p = function(i, j) hole(VecAdd(min, Vec(length1 * (i/count1), length2 * (j/count2), length3)), center) end, c1 = count1, c2 = count2},
                            {p = function(i, j) hole(VecAdd(min, Vec(length1 * (i/count1), 0, length3 * (j/count3))), center) end, c1 = count1, c2 = count3},
                            {p = function(i, j) hole(VecAdd(min, Vec(length1 * (i/count1), length2, length3 * (j/count3))), center) end, c1 = count1, c2 = count3},
                            {p = function(i, j) hole(VecAdd(min, Vec(0, length2 * (i/count2), length3 * (j/count3))), center) end, c1 = count2, c2 = count3},
                            {p = function(i, j) hole(VecAdd(min, Vec(length1, length2 * (i/count2), length3 * (j/count3))), center) end, c1 = count2, c2 = count3}
                        }
    
                        for f=1, #faces do
                            for i=1, faces[f].c1 do
                                for j=1, faces[f].c2 do
                                    faces[f].p(i, j)
                                end
                            end
                        end
                        
                        punchedShape = shape
                        swingTimer = 0.8
                    end
                end
            end
        end

        if swingTimer > 0 then
            local t = Transform()
            t.pos = Vec(0, 0, -swingTimer*2)
            t.rot = QuatEuler(swingTimer*30, 0, 0)
            SetToolTransform(t)

            swingTimer = swingTimer - 0.01
        end
    end
end

function hole(p, center)
    local h, d, n, s = QueryRaycast(p, VecSub(p, center), 0.1)
    if h and not IsBodyDynamic(GetShapeBody(s)) then
        MakeHole(p, stepsize, stepsize, stepsize, true)
    end
end